"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.differ = void 0;
function differ(expected, result) {
    const logs = [];
    const diff = Object.assign(Object.assign({}, detectNewOrUpdatedFiles(expected, result, logs)), detectRemovedFiles(expected, result, logs));
    return {
        diff,
        logs
    };
}
exports.differ = differ;
function detectNewOrUpdatedFiles(expected, result, logs) {
    const diff = {};
    Object.keys(result).forEach((filePath) => {
        const issue = result[filePath];
        if (expected[filePath]) {
            diff[filePath] = diffFileIssue(expected[filePath], issue, logs, filePath);
        }
        else {
            logs.push({ debug: `new file: ${filePath}` });
            diff[filePath] = issue;
        }
    });
    return diff;
}
function diffFileIssue(expected, result, logs, filePath) {
    const fileDiff = {
        branches: 0,
        functions: 0,
        lines: 0,
        statements: 0
    };
    const coverageTypes = Object.keys(fileDiff);
    coverageTypes.forEach((attribute) => {
        const delta = result[attribute] - expected[attribute];
        if (delta < 0) {
            logs.push({
                debug: `"${attribute}" coverage is better in "${filePath}": ${result[attribute]} < ${expected[attribute]}`
            });
        }
        else if (delta > 0) {
            logs.push({
                error: `"${attribute}" coverage is worse in "${filePath}": ${result[attribute]} > ${expected[attribute]}`
            });
        }
        fileDiff[attribute] = delta;
    });
    return fileDiff;
}
function detectRemovedFiles(expected, result, logs) {
    const diff = {};
    Object.keys(expected).forEach((expectedFile) => {
        if (!result[expectedFile]) {
            logs.push({ debug: `${expectedFile} is gone.` });
            diff[expectedFile] = getNegativeIssue(expected[expectedFile]);
        }
    });
    return diff;
}
function getNegativeIssue(issue) {
    return {
        branches: -issue.branches,
        lines: -issue.lines,
        functions: -issue.functions,
        statements: -issue.statements
    };
}
//# sourceMappingURL=differ.js.map